# 
# test_root.rb
# ============
# 
# Test the Root CLI
# 

require 'minitest/autorun'
require 'ptero'
require 'mute'
require 'fileutils'

class TestRoot < MiniTest::Test
  
  def setup
    @klass = Ptero::CLI::Root
    Dir.chdir __dir__
    Mute::IO.capture_stdout do
      @app = Ptero::Application.create('example')
    end
    Dir.chdir @app.dir
  end
  
  def test_new
    Mute::IO.capture_stdout do
      @klass.start(['new', 'boat'])
    end
    app = Ptero::Application.app_for('boat')
    assert app.verify
    assert app.dependencies_installed?
    app.destroy
  end

  def test_seed
    Mute::IO.capture_stdout do
      @klass.start(['seed', 'boat'])
    end
    app = Ptero::Application.app_for('boat')
    assert app.verify
    refute app.dependencies_installed?
    app.destroy
  end

  def test_route_unroute
    assert_raises Ptero::Exception::GeneratorException do
      @klass.start(['route', '/app/path/:number.jpg', 'Photo'])
    end
    
    
    Mute::IO.capture_stdout do
      @app.generate('routes')
      @klass.start(['route', '/app/path/:number.jpg', 'Photo'])
    end
    
    assert @app.routes['/app/path/:number.jpg'] == 'Photo'
    
    Mute::IO.capture_stdout do
      @klass.start(['unroute', '/app/path/:number.jpg'])
      refute @app.routes.has_key? '/app/path/:number.jpg'
      @app.remove('routes')
    end
    
  end
  
  def test_verify
    Mute::IO.capture_stdout do
      Dir.mkdir('notapp')
      refute @klass.start(['verify','notapp'])
      Dir.chdir 'notapp' do
        refute @klass.start(['verify'])
      end
      Dir.rmdir('notapp')
    
      app = Ptero::Application.create('yesapp')
      assert app.verify
      assert @klass.start(['verify','yesapp'])
      Dir.chdir 'yesapp' do
        assert @klass.start(['verify'])
      end
      app.destroy
    end
    
  end
  
  def test_destroy
    Dir.chdir __dir__ do
      Mute::IO.capture_stdout do
        give_stdin do |stdin|
          app = Ptero::Application.create('appp')
          assert app.verify
          stdin.puts 'Yes'
          @klass.start(['destroy','appp'])
          refute app.verify
          refute File.exist? 'appp'
    
          app = Ptero::Application.create('appp')
          assert app.verify
          Dir.chdir 'appp' do
            stdin.puts 'Yes'
            @klass.start(['destroy'])
          end
  
          refute app.verify
          refute File.exist? 'appp'
         
          app = Ptero::Application.create('appp')
          stdin.puts 'not yes'
          @klass.start(['destroy','appp'])
          assert app.verify
          app.destroy
          
        end
      end
    end
  end
  
  def test_composer
    refute @app.has_composer?
    Mute::IO.capture_stdout do
      @klass.start(['composer'])
    end
    assert @app.has_composer?
  end

  def test_install
    Mute::IO.capture_stdout do
      @app.get_composer
    end
    refute @app.dependencies_installed?
    Mute::IO.capture_stdout do
      @klass.start(['install'])
    end
    assert @app.dependencies_installed?
    @app.remove_composer
  end
  
  def test_generate_reload_remove
    ['javascript','view','controller'].each do |str|
      name = 'puppy'
      gen = Ptero::Generator.const_get("#{str.capitalize}Generator".intern).new(name)
      refute gen.generated?
      Mute::IO.capture_stdout do
        @klass.start ['generate',str,name]
        assert gen.generated?
        @klass.start ['reload',str,name]
        assert gen.generated?
        @klass.start ['remove',str,name]
        refute gen.generated?
        @klass.start ['reload',str,name]
        assert gen.generated?
        gen.remove
      end
    end
  end
  
  def test_routes
    refute @app.generated? 'routes'
    pairs = {
      '/phony/1.jpg' => 'Apple',
      'awesome/:number' => 'Bumblebee',
      'eriuvd' => 'SarumanTheWhite'
    }
    Mute::IO.capture_stdout do
      @app.generate_routes
      pairs.each_pair do |key,value|
        @app.route(key,value)
      end
    end
    
    lines = Mute::IO.capture_stdout { @klass.start ['routes'] }.chomp.split("\n")
    
    assert_equal pairs.length, lines.length
    
    lines.each do |line|
      assert line.match(/^\s*([\/\w:\.]+)\s*=>\s*(\w+)Controller,?\s*$/)
      
      assert pairs[$~[1]] == $~[2]
      
    end
  end
  
  def test_version
    # Make sure Ptero::VERSION is the same as the output of $ ptero version
    output = Mute::IO.capture_stdout { @klass.start ['version'] }.chomp
    assert_equal Ptero::VERSION, output
  end


  def teardown
    @app.destroy
    Dir.chdir __dir__
  end
  
  private
  
  def give_stdin
    stdin = $stdin
    $stdin,write = IO.pipe
    yield write
  ensure
    write.close
    $stdin = stdin
  end

  
end